#!/usr/bin/env python3
import uuid

from django.core.management.base import BaseCommand, CommandError

from arkindex.process.models import Repository, Revision, Worker, WorkerType, WorkerVersion


class Command(BaseCommand):
    help = 'Create a fake worker version'

    def add_arguments(self, parser):
        parser.add_argument(
            "--url",
            help="URL of the repository",
            required=True
        )
        parser.add_argument(
            "--name",
            help="Name of the worker",
            required=True
        )
        parser.add_argument(
            "--slug",
            help="Slug of the worker",
            required=True
        )

    def handle(self, url, name, slug, *args, **options):
        versions = WorkerVersion.objects.filter(worker__slug=slug)
        if versions.exists():
            versions_id = list(versions.values_list('id', flat=True).distinct())
            raise CommandError(f"This worker already has versions: {versions_id}")
            return

        repo, _ = Repository.objects.get_or_create(
            url=url,
            defaults={
                "hook_token": str(uuid.uuid4()),
            }
        )

        worker_type, _ = WorkerType.objects.get_or_create(
            slug="classifier",
            defaults={'display_name': "classifier"}
        )

        worker, _ = Worker.objects.get_or_create(
            name=name,
            slug=slug,
            type=worker_type,
            repository=repo
        )

        revision = Revision.objects.create(
            repo_id=repo.id,
            hash=uuid.uuid4().hex,
            message="Fake revision",
            author="Teklia Bot"
        )

        version = WorkerVersion.objects.create(
            worker=worker,
            revision=revision,
            configuration={"configuration": {}},
        )

        self.stdout.write(self.style.SUCCESS(f"Created a worker version: {version.id}"))
